#****************************************************************************
#*                                                                          *
#* NXP LPC2388 Microprocessor startup routine                               *
#*                                                                          *
#****************************************************************************

.equ Mode_FIQ,            0x11                                          @ FIQ Mode
.equ Mode_IRQ,            0x12                                          @ IRQ Mode
.equ Mode_SVC,            0x13                                          @ SuperVisor Mode

.equ FIQ_Stack_Size,      0x00000100                                    @ FIQ Stack Size
.equ IRQ_Stack_Size,      0x00000100                                    @ IRQ Stack Size
.equ SVC_Stack_Size,      0x00000100                                    @ SuperVisor Stack Size

.equ I_Bit,               0x80                                          @ when I bit is set, IRQ is disabled
.equ F_Bit,               0x40                                          @ when F bit is set, FIQ is disabled


# Sample initialization file

         .extern main
         .extern exit

# .text is used instead of .section .text so it works with arm-aout too.

         .text
         .code 32


         .align  0

         .extern __bss_beg__
         .extern __bss_end__
         .extern __stack_end__
         .extern __data_beg__
         .extern __data_end__
         .extern __data+beg_src__
         .extern IRQ_Handler
         .extern FIQ_Handler

         .global start
         .global endless_loop

start:
# Start by setting up a stack
         # Set up the stack pointer to end of bss
         ldr             r3, .LC6
         mov     sp, r3

         sub     sl, sp, #512                                           @ Still assumes 512 bytes below sl

         mov     a2, #0                                                 @ Second arg: fill value
         mov     fp, a2                                                 @ Null frame pointer
         mov     r7, a2                                                 @ Null frame pointer for Thumb

         ldr     r1, .LC1                                               @ First arg: start of memory block
         ldr     r3, .LC2                                               @ Second arg: end of memory block
         subs    r3, r3, r1                                             @ Third arg: length of block
         beq     .end_clear_loop
         mov     r2, #0

.clear_loop:
         strb    r2, [r1], #1
         subs    r3, r3, #1
         bgt     .clear_loop

.end_clear_loop:

         ldr     r1, .LC3                                               @ First arg: start of memory block
         ldr     r2, .LC4                                               @ Second arg: end of memory block
         ldr     r3, .LC5
         subs    r3, r3, r1                                             @ Third arg: length of block
         beq     .end_set_loop

.set_loop:
         ldrb    r4, [r2], #1
         strb    r4, [r1], #1
         subs    r3, r3, #1
         bgt     .set_loop

.end_set_loop:

# Setup Stack for each mode
         ldr     r0, =_sp_base

# Enter FIQ Mode and set its Stack Pointer
         msr     CPSR_c, #Mode_FIQ | I_Bit | F_Bit
         mov     sp, r0
         sub     r0, r0, #FIQ_Stack_Size

# Enter IRQ Mode and set its Stack Pointer
         msr     CPSR_c, #Mode_IRQ | I_Bit | F_Bit
         mov     sp, r0
         sub     r0, r0, #IRQ_Stack_Size

# Enter Supervisor Mode and set its Stack Pointer
         msr     CPSR_c, #Mode_SVC | I_Bit | F_Bit
         mov     sp, r0
         sub     r0, r0, #SVC_Stack_Size

         mov     r0, #0                                                 @ no arguments
         mov     r1, #0                                                 @ no argv either

         bl      main

endless_loop:
         b       endless_loop

# For Thumb, constants must be after the code since only
# positive offsets are supported for PC relative addresses.

         .align 0
.LC1:
.word   __bss_beg__
.LC2:
.word   __bss_end__
.LC3:
.word   __data_beg__
.LC4:
.word   __data_beg_src__
.LC5:
.word   __data_end__
.LC6:
.word   __stack_end__

.section .startup,"ax"
         .code 32
         .align 0

         b       start
         nop
         nop
         nop
         nop
         nop
         b       IRQ_Handler
         b       FIQ_Handler
